@@ -1,3 +1,3 @@
 #define OPEQ MFB_OPEQ_WHITE
 #define MFBIMAGEGLYPHBLT mfbImageGlyphBltWhite
-#include "./mfbimggblt.c"
+#include "../../mfb/mfbimggblt.c"
