@@ -1,3 +1,3 @@
 #define OPEQ MFB_OPEQ_BLACK
 #define MFBPOLYGLYPHBLT mfbPolyGlyphBltBlack
-#include "./mfbplygblt.c"
+#include "../../mfb/mfbplygblt.c"
